#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import os

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.i18n import _


class command:
    def __init__(self):
        pass

    def initialize(self, environment, script_path=""):
        self.env = environment

    def shutdown(self):
        pass

    def get_description(self):
        return _("export the current fenrir clipboard to a file")

    def run(self):
        clipboard_file_path = self.env["runtime"][
            "SettingsManager"
        ].get_setting("general", "clipboardExportPath")
        clipboard_file_path = clipboard_file_path.replace(
            "$user", self.env["general"]["curr_user"]
        )
        clipboard_file_path = clipboard_file_path.replace(
            "$USER", self.env["general"]["curr_user"]
        )
        clipboard_file_path = clipboard_file_path.replace(
            "$User", self.env["general"]["curr_user"]
        )
        clipboard_file = open(clipboard_file_path, "w")
        try:
            if self.env["runtime"]["MemoryManager"].is_index_list_empty(
                "clipboardHistory"
            ):
                self.env["runtime"]["OutputManager"].present_text(
                    _("clipboard empty"), interrupt=True
                )
                return
            clipboard = self.env["runtime"][
                "MemoryManager"
            ].get_index_list_element("clipboardHistory")
            clipboard_file.write(clipboard)
            clipboard_file.close()
            os.chmod(clipboard_file_path, 0o644)
            self.env["runtime"]["OutputManager"].present_text(
                _("clipboard exported to file"), interrupt=True
            )
        except Exception as e:
            self.env["runtime"]["DebugManager"].write_debug_out(
                "export_clipboard_to_file:run: Filepath:"
                + clipboard_file
                + " trace:"
                + str(e),
                debug.DebugLevel.ERROR,
            )

    def set_callback(self, callback):
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core.i18n import _
from fenrirscreenreader.utils import char_utils


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def get_description(self):
        return _("Move review to the character below the current position")

    def run(self):
        cursor_pos = self.env["runtime"][
            "CursorManager"
        ].get_review_or_text_cursor()
        (
            self.env["screen"]["newCursorReview"]["x"],
            self.env["screen"]["newCursorReview"]["y"],
            downChar,
            end_of_screen,
        ) = char_utils.get_down_char(
            self.env["screen"]["newCursorReview"]["x"],
            self.env["screen"]["newCursorReview"]["y"],
            self.env["screen"]["new_content_text"],
        )
        self.env["runtime"]["OutputManager"].present_text(
            downChar,
            interrupt=True,
            ignore_punctuation=True,
            announce_capital=True,
            flush=False,
        )
        if end_of_screen:
            if self.env["runtime"]["SettingsManager"].get_setting_as_bool(
                "review", "end_of_screen"
            ):
                self.env["runtime"]["OutputManager"].present_text(
                    _("end of screen"),
                    interrupt=True,
                    sound_icon="EndOfScreen",
                )

    def set_callback(self, callback):
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.


from fenrirscreenreader.core.i18n import _


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def get_description(self):
        return _("places marks to select text to copy to the clipboard")

    def run(self):
        if not self.env["runtime"]["CursorManager"].is_review_mode():
            self.env["runtime"]["OutputManager"].present_text(
                _("no review cursor"), interrupt=True
            )
            return

        # If in table mode, set header row instead of regular mark
        if self.env["runtime"]["TableManager"].is_table_mode():
            success = self.env["runtime"]["TableManager"].set_header_row_from_cursor()
            if success:
                self.env["runtime"]["OutputManager"].present_text(
                    _("header row set"), sound_icon="PlaceStartMark", interrupt=True
                )
            else:
                self.env["runtime"]["OutputManager"].present_text(
                    _("could not set header row"), interrupt=True
                )
            return

        # Regular mark functionality
        curr_mark = self.env["runtime"]["CursorManager"].set_mark()
        if curr_mark == 1:
            self.env["runtime"]["OutputManager"].present_text(
                _("set mark"), sound_icon="PlaceStartMark", interrupt=True
            )
        elif curr_mark == 2:
            self.env["runtime"]["OutputManager"].present_text(
                _("set mark"), sound_icon="PlaceEndMark", interrupt=True
            )

    def set_callback(self, callback):
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import inspect
import os
import time

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.i18n import _
from fenrirscreenreader.utils import module_utils

currentdir = os.path.dirname(
    os.path.realpath(os.path.abspath(inspect.getfile(inspect.currentframe())))
)
fenrir_path = os.path.dirname(currentdir)


class VmenuManager:
    def __init__(self):
        self.menuDict = {}
        self.curr_index = None
        self.currMenu = ""
        self.active = False
        self.reset = True
        self.useTimeout = True
        self.searchText = ""
        self.lastSearchTime = time.time()

    def initialize(self, environment):
        self.env = environment
        # use default path
        self.defaultVMenuPath = (
            fenrir_path
            + "/commands/vmenu-profiles/"
            + self.env["runtime"]["InputManager"].get_shortcut_type()
        )
        # if there is no user configuration
        if (
            self.env["runtime"]["SettingsManager"].get_setting(
                "menu", "vmenuPath"
            )
            != ""
        ):
            self.defaultVMenuPath = self.env["runtime"][
                "SettingsManager"
            ].get_setting("menu", "vmenuPath")
            if not self.defaultVMenuPath.endswith("/"):
                self.defaultVMenuPath += "/"
            self.defaultVMenuPath += self.env["runtime"][
                "InputManager"
            ].get_shortcut_type()

        self.create_menu_tree()
        self.closeAfterAction = False

    def shutdown(self):
        pass

    def clear_search_text(self):
        self.searchText = ""

    def search_entry(self, value, forceReset=False):
        if self.curr_index is None:
            return ""
        if self.reset or forceReset:
            self.clear_search_text()
        else:
            if self.useTimeout:
                if time.time() - self.lastSearchTime > 1:
                    self.clear_search_text()
        self.searchText += value.upper()
        self.lastSearchTime = time.time()
        start_index = self.get_curr_index()
        while True:
            if not self.next_index():
                return ""
            entry = self.get_current_entry()
            if entry.upper().startswith(self.searchText):
                return entry
            if start_index == self.get_curr_index():
                return ""

    def set_curr_menu(self, currMenu=""):
        self.curr_index = None
        self.currMenu = ""
        if currMenu != "":
            currMenu += " " + _("Menu")
            try:
                t = self.menuDict[currMenu]
                l = list(self.menuDict.keys())
                self.curr_index = [l.index(currMenu)]
            except Exception as e:
                print(e)
                self.currMenu = ""
                self.curr_index = None
                return
            if self.inc_level():
                self.currMenu = currMenu
            else:
                self.currMenu = ""
                self.curr_index = None

    def get_curr_menu(self):
        return self.currMenu

    def get_active(self):
        return self.active

    def toggle_vmenu_mode(self, closeAfterAction=True):
        self.set_active(not self.get_active(), closeAfterAction)

    def set_active(self, active, closeAfterAction=True):
        if self.env["runtime"]["HelpManager"].is_tutorial_mode():
            return
        self.active = active
        if self.active:
            self.closeAfterAction = closeAfterAction
            try:
                self.create_menu_tree()
            except Exception as e:
                print(e)
            try:
                if self.currMenu != "":
                    self.set_curr_menu(self.currMenu)
                if self.curr_index is None:
                    if len(self.menuDict) > 0:
                        self.curr_index = [0]
            except Exception as e:
                print(e)
            try:
                # navigation
                self.env["bindings"][
                    str([1, ["KEY_ESC"]])
                ] = "TOGGLE_VMENU_MODE"
                self.env["bindings"][str([1, ["KEY_UP"]])] = "PREV_VMENU_ENTRY"
                self.env["bindings"][
                    str([1, ["KEY_DOWN"]])
                ] = "NEXT_VMENU_ENTRY"
                self.env["bindings"][
                    str([1, ["KEY_SPACE"]])
                ] = "CURR_VMENU_ENTRY"
                self.env["bindings"][
                    str([1, ["KEY_LEFT"]])
                ] = "DEC_LEVEL_VMENU"
                self.env["bindings"][
                    str([1, ["KEY_RIGHT"]])
                ] = "INC_LEVEL_VMENU"
                self.env["bindings"][
                    str([1, ["KEY_ENTER"]])
                ] = "EXEC_VMENU_ENTRY"
                # search
                self.env["bindings"][str([1, ["KEY_A"]])] = "SEARCH_A"
                self.env["bindings"][str([1, ["KEY_B"]])] = "SEARCH_B"
                self.env["bindings"][str([1, ["KEY_C"]])] = "SEARCH_C"
                self.env["bindings"][str([1, ["KEY_D"]])] = "SEARCH_D"
                self.env["bindings"][str([1, ["KEY_E"]])] = "SEARCH_E"
                self.env["bindings"][str([1, ["KEY_F"]])] = "SEARCH_F"
                self.env["bindings"][str([1, ["KEY_G"]])] = "SEARCH_G"
                self.env["bindings"][str([1, ["KEY_H"]])] = "SEARCH_H"
                self.env["bindings"][str([1, ["KEY_I"]])] = "SEARCH_I"
                self.env["bindings"][str([1, ["KEY_J"]])] = "SEARCH_J"
                self.env["bindings"][str([1, ["KEY_K"]])] = "SEARCH_K"
                self.env["bindings"][str([1, ["KEY_L"]])] = "SEARCH_L"
                self.env["bindings"][str([1, ["KEY_M"]])] = "SEARCH_M"
                self.env["bindings"][str([1, ["KEY_N"]])] = "SEARCH_N"
                self.env["bindings"][str([1, ["KEY_O"]])] = "SEARCH_O"
                self.env["bindings"][str([1, ["KEY_P"]])] = "SEARCH_P"
                self.env["bindings"][str([1, ["KEY_Q"]])] = "SEARCH_Q"
                self.env["bindings"][str([1, ["KEY_R"]])] = "SEARCH_R"
                self.env["bindings"][str([1, ["KEY_S"]])] = "SEARCH_S"
                self.env["bindings"][str([1, ["KEY_T"]])] = "SEARCH_T"
                self.env["bindings"][str([1, ["KEY_U"]])] = "SEARCH_U"
                self.env["bindings"][str([1, ["KEY_V"]])] = "SEARCH_V"
                self.env["bindings"][str([1, ["KEY_W"]])] = "SEARCH_W"
                self.env["bindings"][str([1, ["KEY_X"]])] = "SEARCH_X"
                self.env["bindings"][str([1, ["KEY_Y"]])] = "SEARCH_Y"
                self.env["bindings"][str([1, ["KEY_Z"]])] = "SEARCH_Z"
                # page navigation
                self.env["bindings"][
                    str([1, ["KEY_PAGEUP"]])
                ] = "PAGE_UP_VMENU"
                self.env["bindings"][
                    str([1, ["KEY_PAGEDOWN"]])
                ] = "PAGE_DOWN_VMENU"
            except Exception as e:
                print(e)
        else:
            try:
                self.curr_index = None
                self.env["bindings"] = self.env["runtime"][
                    "SettingsManager"
                ].get_binding_backup()
            except Exception as e:
                self.env["runtime"]["DebugManager"].write_debug_out(
                    "VmenuManager set_active: Error loading binding backup: "
                    + str(e),
                    debug.DebugLevel.ERROR,
                )

    def create_menu_tree(self, resetIndex=True):
        if resetIndex:
            self.curr_index = None
        menu = self.fs_tree_to_dict(self.defaultVMenuPath)
        if menu:
            self.menuDict = menu

        # Add dynamic voice menus
        try:
            from fenrirscreenreader.core.dynamicVoiceMenu import (
                add_dynamic_voice_menus,
            )

            add_dynamic_voice_menus(self)
        except Exception as e:
            print(f"Error adding dynamic voice menus: {e}")

        # Add dynamic keyboard layout menus
        try:
            from fenrirscreenreader.core.dynamicKeyboardLayoutMenu import (
                add_dynamic_keyboard_layout_menus,
            )

            add_dynamic_keyboard_layout_menus(self)
        except Exception as e:
            print(f"Error adding dynamic keyboard layout menus: {e}")

        # index still valid?
        if self.curr_index is not None:
            try:
                r = self.get_value_by_path(self.menuDict, self.curr_index)
                if r == {}:
                    self.curr_index = None
            except Exception as e:
                self.env["runtime"]["DebugManager"].write_debug_out(
                    "VmenuManager create_menu_tree: Error checking menu index validity: "
                    + str(e),
                    debug.DebugLevel.ERROR,
                )
                self.curr_index = None

    def execute_menu(self):
        if self.curr_index is None:
            return
        try:
            command = self.get_value_by_path(self.menuDict, self.curr_index)
            if command is not None:
                command.run()
                if self.closeAfterAction:
                    self.set_active(False)
        except Exception as e:
            try:
                self.inc_level()
                text = self.get_current_entry()
                self.env["runtime"]["OutputManager"].present_text(
                    text, interrupt=True
                )
            except Exception as ex:
                self.env["runtime"]["DebugManager"].write_debug_out(
                    "VmenuManager execute_menu: Error presenting menu text: "
                    + str(ex),
                    debug.DebugLevel.ERROR,
                )

    def inc_level(self):
        if self.curr_index is None:
            return False
        try:
            r = self.get_value_by_path(self.menuDict, self.curr_index + [0])
            if r == {}:
                return False
        except Exception as e:
            self.env["runtime"]["DebugManager"].write_debug_out(
                "VmenuManager inc_level: Error accessing menu path: " + str(e),
                debug.DebugLevel.ERROR,
            )
            return False
        self.curr_index.append(0)
        return True

    def dec_level(self):
        if self.curr_index is None:
            return False
        if self.currMenu != "":
            if len(self.curr_index) <= 2:
                return False
        elif len(self.curr_index) == 1:
            return False
        self.curr_index = self.curr_index[: len(self.curr_index) - 1]
        return True

    def next_index(self):
        if self.curr_index is None:
            return False
        if self.curr_index[len(self.curr_index) - 1] + 1 >= len(
            self.get_nested_by_path(self.menuDict, self.curr_index[:-1])
        ):
            self.curr_index[len(self.curr_index) - 1] = 0
        else:
            self.curr_index[len(self.curr_index) - 1] += 1
        return True

    def get_curr_index(self):
        if self.curr_index is None:
            return 0
        return self.curr_index[len(self.curr_index) - 1]

    def prev_index(self):
        if self.curr_index is None:
            return False
        if self.curr_index[len(self.curr_index) - 1] == 0:
            self.curr_index[len(self.curr_index) - 1] = (
                len(
                    self.get_nested_by_path(
                        self.menuDict, self.curr_index[:-1]
                    )
                )
                - 1
            )
        else:
            self.curr_index[len(self.curr_index) - 1] -= 1
        return True

    def page_up(self):
        if self.curr_index is None:
            return False
        menu_size = len(
            self.get_nested_by_path(self.menuDict, self.curr_index[:-1])
        )
        if menu_size <= 1:
            return False
        jump_size = max(1, int(menu_size * 0.1))  # 10% of menu size, minimum 1
        new_index = self.curr_index[len(self.curr_index) - 1] - jump_size
        if new_index < 0:
            new_index = 0
        self.curr_index[len(self.curr_index) - 1] = new_index
        return True

    def page_down(self):
        if self.curr_index is None:
            return False
        menu_size = len(
            self.get_nested_by_path(self.menuDict, self.curr_index[:-1])
        )
        if menu_size <= 1:
            return False
        jump_size = max(1, int(menu_size * 0.1))  # 10% of menu size, minimum 1
        new_index = self.curr_index[len(self.curr_index) - 1] + jump_size
        if new_index >= menu_size:
            new_index = menu_size - 1
        self.curr_index[len(self.curr_index) - 1] = new_index
        return True

    def get_current_entry(self):
        return self.get_keys_by_path(self.menuDict, self.curr_index)[
            self.curr_index[-1]
        ]

    def fs_tree_to_dict(self, path_):
        for root, dirs, files in os.walk(path_):
            tree = {
                d
                + " "
                + _("Menu"): self.fs_tree_to_dict(os.path.join(root, d))
                for d in dirs
                if not d.startswith("__")
            }
            for f in files:
                try:
                    file_name, file_extension = os.path.splitext(f)
                    file_name = file_name.split("/")[-1]
                    if file_name.startswith("__"):
                        continue
                    # Skip base classes that shouldn't be loaded as commands
                    if file_name.endswith("_base"):
                        continue
                    command = self.env["runtime"]["CommandManager"].load_file(
                        root + "/" + f
                    )
                    tree.update({file_name + " " + _("Action"): command})
                except Exception as e:
                    print(e)
            return tree  # note we discontinue iteration trough os.walk

    def get_nested_by_path(self, complete, path):
        path = path.copy()
        if path != []:
            index = list(complete.keys())[path[0]]
            nested = self.get_nested_by_path(complete[index], path[1:])
            return nested
        else:
            return complete

    def get_keys_by_path(self, complete, path):
        if not isinstance(complete, dict):
            return []
        d = complete
        for i in path[:-1]:
            d = d[list(d.keys())[i]]
        return list(d.keys())

    def get_value_by_path(self, complete, path):
        if not isinstance(complete, dict):
            return complete
        d = complete.copy()
        for i in path:
            d = d[list(d.keys())[i]]
        return d



#!/usr/bin/env python3
"""
Fenrir Syntax Validation Tool

Validates Python syntax across the entire Fenrir codebase without writing
cache files. Designed to catch syntax errors before packaging or releases.

Usage:
    python3 tools/validate_syntax.py                    # Validate all Python files
    python3 tools/validate_syntax.py --fix             # Fix common issues automatically
    python3 tools/validate_syntax.py --check-only      # Exit with non-zero if errors found
"""

import ast
import os
import re
import sys
import argparse
import tempfile
from pathlib import Path


class SyntaxValidator:
    def __init__(self):
        self.errors = []
        self.warnings = []
        self.fixed = []

    def check_fstring_issues(self, content):
        """Check for common f-string formatting issues that cause syntax errors."""
        issues = []
        lines = content.split('\n')

        for i, line in enumerate(lines, 1):
            # Check for f-strings that appear to be broken across lines
            # Pattern: f"some text {
            if re.search(r'f"[^"]*{[^}]*$', line.strip()):
                issues.append(f"Line {i}: Potential broken multiline f-string")

        return issues

    def validate_file(self, filepath):
        """Validate syntax of a single Python file."""
        try:
            with open(filepath, 'r', encoding='utf-8') as f:
                content = f.read()

            # Check for specific f-string issues before AST parsing
            fstring_issues = self.check_fstring_issues(content)
            if fstring_issues:
                # Create a synthetic syntax error for f-string issues
                error_msg = f"F-string issues: {'; '.join(fstring_issues)}"
                self.errors.append((filepath, SyntaxError(error_msg), content))
                return False, content

            # Parse with AST (catches syntax errors)
            ast.parse(content, filename=str(filepath))
            return True, content
            
        except SyntaxError as e:
            error_msg = f"{filepath}:{e.lineno}: {e.msg}"
            self.errors.append((filepath, e, content))
            return False, content
            
        except UnicodeDecodeError as e:
            error_msg = f"{filepath}: Unicode decode error: {e}"
            self.errors.append((filepath, e, None))
            return False, None
            
        except Exception as e:
            error_msg = f"{filepath}: Unexpected error: {e}"
            self.errors.append((filepath, e, None))
            return False, None
    
    def fix_common_issues(self, filepath, content):
        """Attempt to fix common syntax issues automatically."""
        if not content:
            return False, content
            
        original_content = content
        fixed_issues = []
        
        # Fix unterminated f-strings (the main issue from the email)
        lines = content.split('\n')
        modified = False
        
        for i, line in enumerate(lines):
            # Look for f-strings that span multiple lines incorrectly
            if 'f"' in line and line.count('"') % 2 == 1:
                # Check if this looks like a broken multi-line f-string
                indent = len(line) - len(line.lstrip())
                
                # Look ahead for continuation
                j = i + 1
                while j < len(lines) and lines[j].strip():
                    next_line = lines[j]
                    next_indent = len(next_line) - len(next_line.lstrip())
                    
                    # If next line is indented more and has closing brace/quote
                    if (next_indent > indent and 
                        ('"' in next_line or '}' in next_line)):
                        
                        # Try to fix by joining lines properly
                        combined_line = line.rstrip()
                        continuation = next_line.strip()
                        
                        if continuation.startswith(('"', '}', 'str(e)', 'self.', 'fenrirVersion.')):
                            # Fix common patterns
                            if 'str(e)}' in continuation:
                                fixed_line = line.replace('f"', 'f"').rstrip() + '{' + continuation.replace('"', '') + '}'
                            elif continuation.startswith('"'):
                                fixed_line = line + continuation
                            else:
                                fixed_line = line.rstrip() + continuation
                            
                            lines[i] = fixed_line
                            lines[j] = ''  # Remove the continuation line
                            modified = True
                            fixed_issues.append(f"Line {i+1}: Fixed multi-line f-string")
                    break
                    j += 1
        
        if modified:
            content = '\n'.join(lines)
            # Clean up empty lines that were created
            content = '\n'.join(line for line in content.split('\n') if line.strip() or not line)
            
            # Verify the fix worked
            try:
                ast.parse(content, filename=str(filepath))
                self.fixed.append((filepath, fixed_issues))
                return True, content
            except SyntaxError:
                # Fix didn't work, return original
                return False, original_content
        
        return False, content
    
    def scan_directory(self, directory, fix_mode=False):
        """Scan directory for Python files and validate them."""
        python_files = []
        
        # Find all Python files
        for root, dirs, files in os.walk(directory):
            # Skip cache and build directories
            dirs[:] = [d for d in dirs if not d.startswith(('__pycache__', '.git', 'build', 'dist'))]
            
            for file in files:
                if file.endswith('.py'):
                    python_files.append(Path(root) / file)
        
        print(f"Validating {len(python_files)} Python files...")
        
        valid_count = 0
        fixed_count = 0
        
        for filepath in sorted(python_files):
            is_valid, content = self.validate_file(filepath)
            
            if is_valid:
                valid_count += 1
                print(f"✓ {filepath}")
            else:
                print(f"✗ {filepath}")
                
                if fix_mode and content:
                    # Try to fix the file
                    was_fixed, fixed_content = self.fix_common_issues(filepath, content)
                    if was_fixed:
                        # Write the fixed content back
                        with open(filepath, 'w', encoding='utf-8') as f:
                            f.write(fixed_content)
                        print(f"  → Fixed automatically")
                        fixed_count += 1
                        
                        # Re-validate
                        is_valid_now, _ = self.validate_file(filepath)
                        if is_valid_now:
                            valid_count += 1
        
        return valid_count, len(python_files), fixed_count
    
    def print_summary(self, valid_count, total_count, fixed_count=0):
        """Print validation summary."""
        print(f"\n{'='*60}")
        print(f"SYNTAX VALIDATION SUMMARY")
        print(f"{'='*60}")
        print(f"Valid files:     {valid_count}/{total_count}")
        print(f"Invalid files:   {total_count - valid_count}")
        if fixed_count > 0:
            print(f"Auto-fixed:      {fixed_count}")
        
        if self.errors:
            print(f"\nERRORS ({len(self.errors)}):")
            for filepath, error, _ in self.errors:
                if isinstance(error, SyntaxError):
                    print(f"  {filepath}:{error.lineno}: {error.msg}")
                else:
                    print(f"  {filepath}: {error}")
        
        if self.fixed:
            print(f"\nAUTO-FIXES APPLIED ({len(self.fixed)}):")
            for filepath, fixes in self.fixed:
                print(f"  {filepath}:")
                for fix in fixes:
                    print(f"    - {fix}")
        
        success_rate = (valid_count / total_count) * 100 if total_count > 0 else 0
        print(f"\nSuccess rate: {success_rate:.1f}%")
        
        return len(self.errors) == 0


def main():
    parser = argparse.ArgumentParser(description='Validate Python syntax in Fenrir codebase')
    parser.add_argument('--fix', action='store_true', 
                       help='Attempt to fix common syntax issues automatically')
    parser.add_argument('--check-only', action='store_true',
                       help='Exit with non-zero code if syntax errors found')
    parser.add_argument('--directory', default='src/fenrirscreenreader',
                       help='Directory to scan (default: src/fenrirscreenreader)')
    
    args = parser.parse_args()
    
    # Find project root
    script_dir = Path(__file__).parent
    project_root = script_dir.parent
    target_dir = project_root / args.directory
    
    if not target_dir.exists():
        print(f"Error: Directory {target_dir} does not exist")
        sys.exit(1)
    
    print(f"Fenrir Syntax Validator")
    print(f"Target directory: {target_dir}")
    print(f"Fix mode: {'ON' if args.fix else 'OFF'}")
    print()
    
    validator = SyntaxValidator()
    valid_count, total_count, fixed_count = validator.scan_directory(target_dir, fix_mode=args.fix)
    
    all_valid = validator.print_summary(valid_count, total_count, fixed_count)
    
    if args.check_only and not all_valid:
        print(f"\nValidation failed: {total_count - valid_count} files have syntax errors")
        sys.exit(1)
    elif not all_valid:
        print(f"\nWarning: {total_count - valid_count} files have syntax errors")
        if not args.fix:
            print("Run with --fix to attempt automatic fixes")
        sys.exit(1)
    else:
        print(f"\n✓ All {total_count} files have valid syntax")
        sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.soundDriver import sound_driver


class driver(sound_driver):
    def __init__(self):
        sound_driver.__init__(self)

    def initialize(self, environment):
        self.env = environment
        self._initialized = True
        print("Sound Debug Driver: Initialized")

    def shutdown(self):
        if not self._initialized:
            return
        self.cancel()
        self._initialized = False
        print("Sound Debug Driver: Shutdown")

    def play_frequence(
        self, frequence, duration, adjust_volume=0.0, interrupt=True
    ):
        if not self._initialized:
            return
        if interrupt:
            self.cancel()
        print(
            "Sound Debug Driver: play_frequence:"
            + " freq:"
            + str(frequence)
            + " duration:"
            + str(duration)
            + " adjust_volume:"
            + str(adjust_volume)
        )
        print("Sound Debug Driver: -----------------------------------")

    def play_sound_file(self, file_path, interrupt=True):
        if not self._initialized:
            return
        if interrupt:
            self.cancel()
        print("Sound Debug Driver: play_sound_file:" + str(file_path))
        print("Sound Debug Driver: -----------------------------------")

    def cancel(self):
        if not self._initialized:
            return
        print("Sound Debug Driver: Cancel")

    def set_callback(self, callback):
        if not self._initialized:
            return
        print("Sound Debug Driver: set_callback")

    def set_volume(self, volume):
        if not self._initialized:
            return
        self.volume = volume
        print("Sound Debug Driver: set_volume:" + str(self.volume))
